﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.UI;

namespace UrlRewriting
{
    public class UrlRewriteHandlerFactory : IHttpHandlerFactory
    {
        IHttpHandler IHttpHandlerFactory.GetHandler(HttpContext context, string requestType, string url, string pathTranslated)
        {
            IHttpHandler pageHandler = null;
            
            NameValueCollection rulesSection =
                ConfigurationManager.GetSection("rewritingEngine") as NameValueCollection;

            foreach (string key in rulesSection)
            {
                string pattern = Utils.ResolveUrl(key);
                string mappedUrl = rulesSection[key];

                Regex expression = new Regex(pattern, RegexOptions.IgnoreCase);

                if (expression.IsMatch(url))
                {
                    // reg-ex match has been made - duplicate Control.ResolveClientUrl functionality
                    string newUrl = Utils.ResolveUrl(expression.Replace(url, mappedUrl));
                    int pos = newUrl.IndexOf("?");
                    string urlSansQuerystring = newUrl;
                    string querystring = string.Empty;
                    if(pos > -1)
                    {
                        urlSansQuerystring = newUrl.Substring(0, pos);
                        querystring = newUrl.Substring(pos + 1);
                    }
                    string newPath = context.Server.MapPath(urlSansQuerystring);

                    pageHandler = PageParser.GetCompiledPageInstance(urlSansQuerystring, newPath, context);
                    context.RewritePath(urlSansQuerystring, string.Empty, querystring);
                    break;
                }
            }

            // default Page handler as normally served by the PageHandlerFactory
            if (pageHandler == null)
                pageHandler = PageParser.GetCompiledPageInstance(url, pathTranslated, context);

            return pageHandler;
        }

        void IHttpHandlerFactory.ReleaseHandler(IHttpHandler handler)
        {
        }
    }
}
